@@ -20,12 +20,6 @@
 
 #include "display.h"
 
-#ifdef GNOME
-#include <gnome.h>
-#endif
-#include "gtkhwrapbox.h"
-#include "intl.h"
-#include "interface.h"
 #include "menus.h"
 #include "disp_callbacks.h"
 #include "tool.h"
@@ -36,6 +30,29 @@
 #include "linewidth_area.h"
 #include "attributes.h"
 
+/* Integrated UI Constants */
+#define  DIA_MAIN_WINDOW   "dia-main-window"
+#define  DIA_MAIN_NOTEBOOK "dia-main-notebook"
+
+/* Distributed UI Constants */
+#define  DIA_TOOLBOX       "dia-toolbox"
+
+void create_integrated_ui (void);
+
+gboolean integrated_ui_main_toolbar_is_showing (void);
+void     integrated_ui_main_toolbar_show (void);
+void     integrated_ui_main_toolbar_hide (void);
+
+gboolean integrated_ui_main_statusbar_is_showing (void);
+void     integrated_ui_main_statusbar_show (void);
+void     integrated_ui_main_statusbar_hide (void);
+
+gboolean integrated_ui_layer_view_is_showing (void);
+void     integrated_ui_layer_view_show (void);
+void     integrated_ui_layer_view_hide (void);
+
+int is_integrated_ui (void);
+
 void create_display_shell(DDisplay *ddisp,
 			  int width, int height,
 			  char *title, int use_mbar, int top_level_window);
@@ -47,12 +64,13 @@ void toolbox_hide(void);
 GtkWidget *interface_get_toolbox_shell(void);
 
 void tool_select_callback(GtkWidget *widget, gpointer data);
-
+void create_integrated_ui (void);
 void create_tree_window(void);
 
-GtkWidget *popup_shell;
 void create_sheets(GtkWidget *parent);
-GtkWidget *modify_tool_button;
+extern GtkWidget *modify_tool_button;
+
+void view_zoom_set (float zoom_factor); /* zoom_factor is 10 * percentage */
 
 typedef struct _ToolButton ToolButton;
 
@@ -70,7 +88,8 @@ struct _ToolButton
 {
   gchar **icon_data;
   char  *tool_desc;
-  char  *menuitem_name;
+  char	*tool_accelerator;
+  char  *action_name;
   ToolButtonData callback_data;
 };
 
@@ -81,6 +100,10 @@ extern gchar *interface_current_sheet_name;
 void tool_select_update (GtkWidget *w, gpointer   data);
 void fill_sheet_menu(void);
 
+#ifndef GTK_DISABLE_DEPRECATED
 extern GtkTooltips *tool_tips;
+#endif
+
+void close_notebook_page_callback (GtkButton *button, gpointer user_data);
 
 #endif /* INTERFACE_H */
