@@ -17,15 +17,18 @@
  */
 #include <config.h>
 
+#define _BSD_SOURCE 1 /* to get the prototype for fchmod() from sys/stat.h */
+#define _POSIX_C_SOURCE 2 /* so we get fdopen declared even when compiling with -ansi */
+#include <sys/stat.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <stdlib.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
+
 #include <glib.h>
 #include <glib/gstdio.h> /* g_access() and friends */
 #include <errno.h>
@@ -50,6 +53,7 @@
 #include "preferences.h"
 #include "diapagelayout.h"
 #include "autosave.h"
+#include "newgroup.h"
 
 #ifdef G_OS_WIN32
 #include <io.h>
@@ -62,7 +66,8 @@ static void GHFuncUnknownObjects(gpointer key,
 				 gpointer user_data);
 static GList *read_objects(xmlNodePtr objects,
 			   GHashTable *objects_hash,
-			   const char *filename, DiaObject *parent);
+			   const char *filename, DiaObject *parent,
+			   GHashTable *unknown_objects_hash);
 static void hash_free_string(gpointer       key,
 			     gpointer       value,
 			     gpointer       user_data);
@@ -109,7 +114,8 @@ GHFuncUnknownObjects(gpointer key,
  */
 static GList *
 read_objects(xmlNodePtr objects, 
-             GHashTable *objects_hash,const char *filename, DiaObject *parent)
+             GHashTable *objects_hash,const char *filename, DiaObject *parent,
+	     GHashTable *unknown_objects_hash)
 {
   GList *list;
   DiaObjectType *type;
@@ -119,13 +125,8 @@ read_objects(xmlNodePtr objects,
   char *versionstr;
   char *id;
   int version;
-  GHashTable* unknown_hash;
-  GString*    unknown_str;
   xmlNodePtr child_node;
 
-  unknown_hash = g_hash_table_new(g_str_hash, g_str_equal);
-  unknown_str  = g_string_new("Unknown types while reading diagram file");
-
   list = NULL;
 
   obj_node = objects->xmlChildrenNode;
@@ -138,10 +139,10 @@ read_objects(xmlNodePtr objects,
 
     if (!obj_node) break;
 
-    if (strcmp(obj_node->name, "object")==0) {
-      typestr = xmlGetProp(obj_node, "type");
-      versionstr = xmlGetProp(obj_node, "version");
-      id = xmlGetProp(obj_node, "id");
+    if (xmlStrcmp(obj_node->name, (const xmlChar *)"object")==0) {
+      typestr = (char *) xmlGetProp(obj_node, (const xmlChar *)"type");
+      versionstr = (char *) xmlGetProp(obj_node, (const xmlChar *)"version");
+      id = (char *) xmlGetProp(obj_node, (const xmlChar *)"id");
       
       version = 0;
       if (versionstr != NULL) {
@@ -152,8 +153,9 @@ read_objects(xmlNodePtr objects,
       type = object_get_type((char *)typestr);
       
       if (!type) {
-	if (NULL == g_hash_table_lookup(unknown_hash, typestr))
-	    g_hash_table_insert(unknown_hash, g_strdup(typestr), 0);
+	if (g_utf8_validate (typestr, -1, NULL) &&
+	    NULL == g_hash_table_lookup(unknown_objects_hash, typestr))
+	    g_hash_table_insert(unknown_objects_hash, g_strdup(typestr), 0);
       }
       else
       {
@@ -172,9 +174,9 @@ read_objects(xmlNodePtr objects,
 
         while(child_node)
         {
-          if (strcmp(child_node->name, "children") == 0)
+          if (xmlStrcmp(child_node->name, (const xmlChar *)"children") == 0)
           {
-	    GList *children_read = read_objects(child_node, objects_hash, filename, obj);
+	    GList *children_read = read_objects(child_node, objects_hash, filename, obj, unknown_objects_hash);
             list = g_list_concat(list, children_read);
             break;
           }
@@ -183,27 +185,55 @@ read_objects(xmlNodePtr objects,
       }
       if (typestr) xmlFree(typestr);
       if (id) xmlFree (id);
-    } else if (   strcmp(obj_node->name, "group")==0
+    } else if (xmlStrcmp(obj_node->name, (const xmlChar *)"group")==0
                && obj_node->children) {
       /* don't create empty groups */
-      obj = group_create(read_objects(obj_node, objects_hash, filename, NULL));
+      obj = group_create(read_objects(obj_node, objects_hash, filename, NULL, unknown_objects_hash));
+#ifdef USE_NEWGROUP
+      /* Old group objects had objects recursively inside them.  Since
+       * objects are now done with parenting, we need to extract those objects,
+       * make a newgroup object, set parent-child relationships, and add
+       * all of them to the diagram.
+       */
+      {
+	DiaObject *newgroup;
+	GList *objects;
+	Point lower_right;
+	type = object_get_type("Misc - NewGroup");
+	lower_right = obj->position;
+	newgroup = type->ops->create(&lower_right,
+				     NULL,
+				     NULL,
+				     NULL);
+	list = g_list_append(list, newgroup);
+	
+	for (objects = group_objects(obj); objects != NULL; objects = g_list_next(objects)) {
+	  DiaObject *subobj = (DiaObject *) objects->data;
+	  list = g_list_append(list, subobj);
+	  subobj->parent = newgroup;
+          newgroup->children = g_list_append(newgroup->children, subobj);
+	  if (subobj->bounding_box.right > lower_right.x) {
+	    lower_right.x = subobj->bounding_box.right;
+	  }
+	  if (subobj->bounding_box.bottom > lower_right.y) {
+	    lower_right.y = subobj->bounding_box.bottom;
+	  }
+	}
+	newgroup->ops->move_handle(newgroup, newgroup->handles[7],
+			       &lower_right, NULL,
+			       HANDLE_MOVE_CREATE_FINAL, 0);
+	/* We've used the info from the old group, destroy it */
+	group_destroy_shallow(obj);
+      }
+#else
       list = g_list_append(list, obj);
+#endif
     } else {
       /* silently ignore other nodes */
     }
 
     obj_node = obj_node->next;
   }
-  /* show all the unknown types in one message */
-  if (0 < g_hash_table_size(unknown_hash)) {
-    g_hash_table_foreach(unknown_hash,
-			 GHFuncUnknownObjects,
-			 unknown_str);
-    message_error("%s", unknown_str->str);
-  }
-  g_hash_table_destroy(unknown_hash);
-  g_string_free(unknown_str, TRUE);
-
   return list;
 }
 
@@ -226,15 +256,21 @@ read_connections(GList *objects, xmlNodePtr layer_node,
   while ((list != NULL) && (obj_node != NULL)) {
     DiaObject *obj = (DiaObject *) list->data;
 
-    while (obj_node && xmlIsBlankNode(obj_node)) obj_node = obj_node->next;
+    /* the obj and there node must stay in sync to properly setup connections */
+    while (obj_node && (xmlIsBlankNode(obj_node) || XML_COMMENT_NODE == obj_node->type)) 
+      obj_node = obj_node->next;
     if (!obj_node) break;
     
     if IS_GROUP(obj) {
       read_connections(group_objects(obj), obj_node, objects_hash);
     } else {
+      gboolean broken = FALSE;
+      /* an invalid bounding box is a good sign for some need of corrections */
+      gboolean wants_update = obj->bounding_box.right >= obj->bounding_box.left 
+                           || obj->bounding_box.top >= obj->bounding_box.bottom;
       connections = obj_node->xmlChildrenNode;
       while ((connections!=NULL) &&
-	     (strcmp(connections->name, "connections")!=0))
+	     (xmlStrcmp(connections->name, (const xmlChar *)"connections")!=0))
 	connections = connections->next;
       if (connections != NULL) {
 	connection = connections->xmlChildrenNode;
@@ -244,9 +280,9 @@ read_connections(GList *objects, xmlNodePtr layer_node,
             connection = connection->next;
             continue;
           }
-	  handlestr = xmlGetProp(connection, "handle");
-	  tostr = xmlGetProp(connection, "to");
-	  connstr = xmlGetProp(connection, "connection");
+	  handlestr = (char * )xmlGetProp(connection, (const xmlChar *)"handle");
+	  tostr = (char *) xmlGetProp(connection, (const xmlChar *)"to");
+ 	  connstr = (char *) xmlGetProp(connection, (const xmlChar *)"connection");
 	  
 	  handle = atoi(handlestr);
 	  conn = strtol(connstr, &donestr, 10);
@@ -259,9 +295,12 @@ read_connections(GList *objects, xmlNodePtr layer_node,
 	  if (to == NULL) {
 	    message_error(_("Error loading diagram.\n"
 			    "Linked object not found in document."));
+	    broken = TRUE;
 	  } else if (handle < 0 || handle >= obj->num_handles) {
 	    message_error(_("Error loading diagram.\n"
-			    "connection handle does not exist."));
+			    "connection handle %d does not exist on '%s'."),
+			    handle, to->type->name);
+	    broken = TRUE;
 	  } else {
 	    if (conn == -1) { /* Find named connpoint */
 	      int i;
@@ -276,10 +315,20 @@ read_connections(GList *objects, xmlNodePtr layer_node,
 	    if (conn >= 0 && conn < to->num_connections) {
 	      object_connect(obj, obj->handles[handle],
 			     to->connections[conn]);
+	      /* force an update on the connection, helpful with (incomplete) generated files */
+	      if (wants_update) {
+	        obj->handles[handle]->pos = 
+	          to->connections[conn]->last_pos = to->connections[conn]->pos;
+#if 0
+	        obj->ops->move_handle(obj, obj->handles[handle], &to->connections[conn]->pos,
+				      to->connections[conn], HANDLE_MOVE_CONNECTED,0);
+#endif
+	      }
 	    } else {
 	      message_error(_("Error loading diagram.\n"
-			      "connection point %s does not exist."),
-			    connstr);
+			      "connection point %d does not exist on '%s'."),
+			    conn, to->type->name);
+	      broken = TRUE;
 	    }
 	  }
 
@@ -289,16 +338,30 @@ read_connections(GList *objects, xmlNodePtr layer_node,
 
 	  connection = connection->next;
 	}
+        /* Fix positions of the connection object for (de)generated files.
+         * Only done for the last point connected otherwise the intermediate posisitions
+         * may screw the auto-routing algorithm.
+         */
+        if (!broken && obj && obj->ops->set_props && wants_update) {
+	  /* called for it's side-effect of update_data */
+	  obj->ops->move(obj,&obj->position);
+
+	  for (handle = 0; handle < obj->num_handles; ++handle) {
+	    if (obj->handles[handle]->connected_to)
+	      obj->ops->move_handle(obj, obj->handles[handle], &obj->handles[handle]->pos,
+				    obj->handles[handle]->connected_to, HANDLE_MOVE_CONNECTED,0);
+	  }
+	}
       }
     }
     
     /* Now set up parent relationships. */
     connections = obj_node->xmlChildrenNode;
     while ((connections!=NULL) &&
-	   (strcmp(connections->name, "childnode")!=0))
+	   (xmlStrcmp(connections->name, (const xmlChar *)"childnode")!=0))
       connections = connections->next;
     if (connections != NULL) {
-      tostr = xmlGetProp(connections, "parent");
+      tostr = (char *)xmlGetProp(connections, (const xmlChar *)"parent");
       if (tostr) {
 	obj->parent = g_hash_table_lookup(objects_hash, tostr);
 	if (obj->parent == NULL) {
@@ -326,7 +389,7 @@ hash_free_string(gpointer       key,
 static xmlNodePtr
 find_node_named (xmlNodePtr p, const char *name)
 {
-  while (p && 0 != strcmp(p->name, name))
+  while (p && 0 != xmlStrcmp(p->name, (xmlChar *)name))
     p = p->next;
   return p;
 }
@@ -346,6 +409,9 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   xmlNsPtr namespace;
   gchar firstchar;
   Diagram *diagram = DIA_IS_DIAGRAM (data) ? DIA_DIAGRAM (data) : NULL;
+  Layer *active_layer = NULL;
+  GHashTable* unknown_objects_hash = g_hash_table_new(g_str_hash, g_str_equal);
+
 
   if (g_file_test (filename, G_FILE_TEST_IS_DIR)) {
     message_error(_("You must specify a file, not a directory.\n"));
@@ -385,8 +451,8 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
     return FALSE;
   }
 
-  namespace = xmlSearchNs(doc, doc->xmlRootNode, "dia");
-  if (strcmp (doc->xmlRootNode->name, "diagram") || (namespace == NULL)){
+  namespace = xmlSearchNs(doc, doc->xmlRootNode, (const xmlChar *)"dia");
+  if (xmlStrcmp (doc->xmlRootNode->name, (const xmlChar *)"diagram") || (namespace == NULL)){
     message_error(_("Error loading diagram %s.\nNot a Dia file."), 
 		  dia_message_filename(filename));
     xmlFreeDoc (doc);
@@ -552,6 +618,7 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   while (layer_node != NULL) {
     gchar *name;
     char *visible;
+    char *active;
     
     if (xmlIsBlankNode(layer_node)) {
       layer_node = layer_node->next;
@@ -560,31 +627,41 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
 
     if (!layer_node) break;
     
-    name = (char *)xmlGetProp(layer_node, "name");
+    name = (char *)xmlGetProp(layer_node, (const xmlChar *)"name");
     if (!name) break; /* name is mandatory */
 
-    visible = (char *)xmlGetProp(layer_node, "visible");
-
     layer = new_layer(g_strdup(name), data);
     if (name) xmlFree(name);
 
     layer->visible = FALSE;
-    if ((visible) && (strcmp(visible, "true")==0))
-      layer->visible = TRUE;
-    if (visible) xmlFree(visible);
-
+    visible = (char *)xmlGetProp(layer_node, (const xmlChar *)"visible");
+    if (visible) {
+      if (strcmp(visible, "true")==0)
+        layer->visible = TRUE;
+      xmlFree(visible);
+    }
     /* Read in all objects: */
     
-    list = read_objects(layer_node, objects_hash, filename, NULL);
+    list = read_objects(layer_node, objects_hash, filename, NULL, unknown_objects_hash);
     layer_add_objects (layer, list);
     read_connections( list, layer_node, objects_hash);
 
     data_add_layer(data, layer);
 
+    active = (char *)xmlGetProp(layer_node, (const xmlChar *)"active");
+    if (active) {
+      if (strcmp(active, "true")==0)
+        active_layer = layer;
+      xmlFree(active);
+    }
+
     layer_node = layer_node->next;
   }
 
-  data->active_layer = (Layer *) g_ptr_array_index(data->layers, 0);
+  if (!active_layer)
+    data->active_layer = (Layer *) g_ptr_array_index(data->layers, 0);
+  else
+    data_set_active_layer(data, active_layer);
 
   xmlFreeDoc(doc);
   
@@ -597,7 +674,17 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
                      "A valid Dia file defines at least one layer."),
 		     dia_message_filename(filename));
     return FALSE;
+  } else if (0 < g_hash_table_size(unknown_objects_hash)) {
+    GString*    unknown_str = g_string_new("Unknown types while reading diagram file");
+
+    /* show all the unknown types in one message */
+    g_hash_table_foreach(unknown_objects_hash,
+			 GHFuncUnknownObjects,
+			 unknown_str);
+    message_warning("%s", unknown_str->str);
+    g_string_free(unknown_str, TRUE);
   }
+  g_hash_table_destroy(unknown_objects_hash);
 
   return TRUE;
 }
@@ -622,18 +709,18 @@ write_objects(GList *objects, xmlNodePtr objects_node,
     }
 
     if (IS_GROUP(obj) && group_objects(obj) != NULL) {
-      group_node = xmlNewChild(objects_node, NULL, "group", NULL);
+      group_node = xmlNewChild(objects_node, NULL, (const xmlChar *)"group", NULL);
       write_objects(group_objects(obj), group_node,
 		    objects_hash, obj_nr, filename);
     } else {
-      obj_node = xmlNewChild(objects_node, NULL, "object", NULL);
+      obj_node = xmlNewChild(objects_node, NULL, (const xmlChar *)"object", NULL);
     
-      xmlSetProp(obj_node, "type", obj->type->name);
+      xmlSetProp(obj_node, (const xmlChar *)"type", (xmlChar *)obj->type->name);
       g_snprintf(buffer, 30, "%d", obj->type->version);
-      xmlSetProp(obj_node, "version", buffer);
+      xmlSetProp(obj_node, (const xmlChar *)"version", (xmlChar *)buffer);
       
       g_snprintf(buffer, 30, "O%d", *obj_nr);
-      xmlSetProp(obj_node, "id", buffer);
+      xmlSetProp(obj_node, (const xmlChar *)"id", (xmlChar *)buffer);
 
       (*obj->type->ops->save)(obj, obj_node, filename);
 
@@ -709,25 +796,25 @@ write_connections(GList *objects, xmlNodePtr layer_node,
 	  }
 	  
 	  if (connections == NULL)
-	    connections = xmlNewChild(obj_node, NULL, "connections", NULL);
+	    connections = xmlNewChild(obj_node, NULL, (const xmlChar *)"connections", NULL);
 	  
-	  connection = xmlNewChild(connections, NULL, "connection", NULL);
+	  connection = xmlNewChild(connections, NULL, (const xmlChar *)"connection", NULL);
 	  /* from what handle on this object*/
 	  g_snprintf(buffer, 30, "%d", i);
-	  xmlSetProp(connection, "handle", buffer);
+	  xmlSetProp(connection, (const xmlChar *)"handle", (xmlChar *)buffer);
 	  /* to what object */
 	  g_snprintf(buffer, 30, "O%d",
 		     GPOINTER_TO_INT(g_hash_table_lookup(objects_hash,
 							 other_obj)));
 
-	  xmlSetProp(connection, "to", buffer);
+	  xmlSetProp(connection, (const xmlChar *)"to", (xmlChar *) buffer);
 	  /* to what connection_point on that object */
 	  if (other_obj->connections[con_point_nr]->name != NULL) {
 	    g_snprintf(buffer, 30, "%s", other_obj->connections[con_point_nr]->name);
 	  } else {
 	    g_snprintf(buffer, 30, "%d", con_point_nr);
 	  }
-	  xmlSetProp(connection, "connection", buffer);
+	  xmlSetProp(connection, (const xmlChar *)"connection", (xmlChar *) buffer);
 	}
       }
     }
@@ -737,8 +824,8 @@ write_connections(GList *objects, xmlNodePtr layer_node,
       xmlNodePtr parent;
       g_snprintf(buffer, 30, "O%d",
 		 GPOINTER_TO_INT(g_hash_table_lookup(objects_hash, other_obj)));
-      parent = xmlNewChild(obj_node, NULL, "childnode", NULL);
-      xmlSetProp(parent, "parent", buffer);
+      parent = xmlNewChild(obj_node, NULL, (const xmlChar *)"childnode", NULL);
+      xmlSetProp(parent, (const xmlChar *)"parent", (xmlChar *)buffer);
     }
     
     list = g_list_next(list);
@@ -758,22 +845,22 @@ diagram_data_write_doc(DiagramData *data, const char *filename)
   GHashTable *objects_hash;
   gboolean res;
   int obj_nr;
-  int i;
+  guint i;
   Layer *layer;
   AttributeNode attr;
   xmlNs *name_space;
   Diagram *diagram = DIA_IS_DIAGRAM (data) ? DIA_DIAGRAM (data) : NULL;
 
-  doc = xmlNewDoc("1.0");
-  doc->encoding = xmlStrdup("UTF-8");
-  doc->xmlRootNode = xmlNewDocNode(doc, NULL, "diagram", NULL);
+  doc = xmlNewDoc((const xmlChar *)"1.0");
+  doc->encoding = xmlStrdup((const xmlChar *)"UTF-8");
+  doc->xmlRootNode = xmlNewDocNode(doc, NULL, (const xmlChar *)"diagram", NULL);
 
   name_space = xmlNewNs(doc->xmlRootNode, 
-                        DIA_XML_NAME_SPACE_BASE,
-			"dia");
+                        (const xmlChar *)DIA_XML_NAME_SPACE_BASE,
+			(const xmlChar *)"dia");
   xmlSetNs(doc->xmlRootNode, name_space);
 
-  tree = xmlNewChild(doc->xmlRootNode, name_space, "diagramdata", NULL);
+  tree = xmlNewChild(doc->xmlRootNode, name_space, (const xmlChar *)"diagramdata", NULL);
   
   attr = new_attribute((ObjectNode)tree, "background");
   data_add_color(attr, &data->bg_color);
@@ -837,15 +924,18 @@ diagram_data_write_doc(DiagramData *data, const char *filename)
   obj_nr = 0;
 
   for (i = 0; i < data->layers->len; i++) {
-    layer_node = xmlNewChild(doc->xmlRootNode, name_space, "layer", NULL);
+    layer_node = xmlNewChild(doc->xmlRootNode, name_space, (const xmlChar *)"layer", NULL);
     layer = (Layer *) g_ptr_array_index(data->layers, i);
-    xmlSetProp(layer_node, "name", layer->name);
+    xmlSetProp(layer_node, (const xmlChar *)"name", (xmlChar *)layer->name);
 
     if (layer->visible)
-      xmlSetProp(layer_node, "visible", "true");
+      xmlSetProp(layer_node, (const xmlChar *)"visible", (const xmlChar *)"true");
     else
-      xmlSetProp(layer_node, "visible", "false");
-    
+      xmlSetProp(layer_node, (const xmlChar *)"visible", (const xmlChar *)"false");
+
+    if (layer == data->active_layer)
+      xmlSetProp(layer_node, (const xmlChar *)"active", (const xmlChar *)"true");
+      
     write_objects(layer->objects, layer_node,
 		  objects_hash, &obj_nr, filename);
   
@@ -888,25 +978,37 @@ diagram_data_raw_save(DiagramData *data, const char *filename)
  * indicated, an error message will already have been given to the user.
  */
 static int
-diagram_data_save(DiagramData *data, const char *filename)
+diagram_data_save(DiagramData *data, const char *user_filename)
 {
   FILE *file;
-  char *bakname,*tmpname,*dirname,*p;
+  char *bakname=NULL,*tmpname=NULL,*dirname=NULL,*p;
+  char *filename = (char *)user_filename;
   int mode,_umask;
   int fildes;
-  int ret;
+  int ret = 0;
 
   /* Once we depend on GTK 2.8+, we can use these tests. */
-#if GLIB_CHECK_VERSION(2,8,0)
+#if GLIB_CHECK_VERSION(2,8,0) && !defined G_OS_WIN32
   /* Check that we're allowed to write to the target file at all. */
-  if (g_file_test(filename, G_FILE_TEST_EXISTS) &&
-      g_access(filename, W_OK) != 0) {
-    message_error(_("No allowed to write to output file %s\n"), 
+  /* not going to work with 'My Docments' - read-only but still useable, see bug #504469 */
+  if (   g_file_test(filename, G_FILE_TEST_EXISTS)
+      && g_access(filename, W_OK) != 0) {
+    message_error(_("Not allowed to write to output file %s\n"), 
 		  dia_message_filename(filename));
-    return FALSE;
+    goto CLEANUP;
   }
 #endif
 
+  if (g_file_test(user_filename, G_FILE_TEST_IS_SYMLINK)) {
+    GError *error = NULL;
+    filename = g_file_read_link(user_filename, &error);
+    if (!filename) {
+      message_error("%s", error->message);
+      g_error_free(error);
+      goto CLEANUP;
+    }
+  }
+
   /* build the temporary and backup file names */
   dirname = g_strdup(filename);
   p = strrchr((char *)dirname,G_DIR_SEPARATOR);
@@ -919,13 +1021,13 @@ diagram_data_save(DiagramData *data, const char *filename)
   tmpname = g_strconcat(dirname,"__diaXXXXXX",NULL);
   bakname = g_strconcat(filename,"~",NULL);
 
-#if GLIB_CHECK_VERSION(2,8,0)
+#if GLIB_CHECK_VERSION(2,8,0) && !defined G_OS_WIN32
   /* Check that we can create the other files */
-  if (g_file_test(dirname, G_FILE_TEST_EXISTS) &&
-      g_access(dirname, W_OK) != 0) {
-    message_error(_("No allowed to write temporary files in %s\n"), 
+  if (   g_file_test(dirname, G_FILE_TEST_EXISTS) 
+      && g_access(dirname, W_OK) != 0) {
+    message_error(_("Not allowed to write temporary files in %s\n"), 
 		  dia_message_filename(dirname));
-    return FALSE;
+    goto CLEANUP;
   }
 #endif
 
@@ -947,7 +1049,7 @@ diagram_data_save(DiagramData *data, const char *filename)
   if (file==NULL) {
     message_error(_("Can't open output file %s: %s\n"), 
 		  dia_message_filename(tmpname), strerror(errno));
-    return FALSE;
+    goto CLEANUP;
   }
   fclose(file);
 
@@ -959,10 +1061,7 @@ diagram_data_save(DiagramData *data, const char *filename)
     message_error(_("Internal error %d writing file %s\n"), 
 		  ret, dia_message_filename(tmpname));
     g_unlink(tmpname);
-    g_free(tmpname);
-    g_free(dirname);
-    g_free(bakname);
-    return FALSE;
+    goto CLEANUP;
   }
   /* save succeeded. We kill the old backup file, move the old file into 
      backup, and the temp file into the new saved file. */
@@ -974,6 +1073,9 @@ diagram_data_save(DiagramData *data, const char *filename)
 		  dia_message_filename(filename), 
 		  dia_message_filename(filename), strerror(errno));
   }
+CLEANUP:
+  if (filename != user_filename)
+    g_free(filename);
   g_free(tmpname);
   g_free(dirname);
   g_free(bakname);
