@@ -50,10 +50,14 @@ struct _SmallPackage {
   char *st_stereotype;
   TextAttributes attrs;
 
+  real line_width;
   Color line_color;
   Color fill_color;
 };
 
+/* The old border width, kept for compatibility with dia files created with
+ * older versions.
+ */
 #define SMALLPACKAGE_BORDERWIDTH 0.1
 #define SMALLPACKAGE_TOPHEIGHT 0.9
 #define SMALLPACKAGE_TOPWIDTH 1.5
@@ -109,15 +113,18 @@ static ObjectOps smallpackage_ops = {
   (MoveFunc)            smallpackage_move,
   (MoveHandleFunc)      smallpackage_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
-  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ApplyPropertiesDialogFunc) object_apply_props_from_dialog,
   (ObjectMenuFunc)      NULL,
   (DescribePropsFunc)   smallpackage_describe_props,
   (GetPropsFunc)        smallpackage_get_props,
   (SetPropsFunc)        smallpackage_set_props,
+  (TextEditFunc) 0,
+  (ApplyPropertiesListFunc) object_apply_props,
 };
 
 static PropDescription smallpackage_props[] = {
   ELEMENT_COMMON_PROPERTIES,
+  PROP_STD_LINE_WIDTH_OPTIONAL,
   PROP_STD_LINE_COLOUR_OPTIONAL, 
   PROP_STD_FILL_COLOUR_OPTIONAL, 
   { "stereotype", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
@@ -140,12 +147,13 @@ smallpackage_describe_props(SmallPackage *smallpackage)
 
 static PropOffset smallpackage_offsets[] = {
   ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  { PROP_STDNAME_LINE_WIDTH, PROP_STDTYPE_LINE_WIDTH, offsetof(SmallPackage, line_width) },
   {"line_colour", PROP_TYPE_COLOUR, offsetof(SmallPackage, line_color) },
   {"fill_colour", PROP_TYPE_COLOUR, offsetof(SmallPackage, fill_color) },
   {"stereotype", PROP_TYPE_STRING, offsetof(SmallPackage , stereotype) },
   {"text",PROP_TYPE_TEXT,offsetof(SmallPackage,text)},
   {"text_font",PROP_TYPE_FONT,offsetof(SmallPackage,attrs.font)},
-  {"text_height",PROP_TYPE_REAL,offsetof(SmallPackage,attrs.height)},
+  {PROP_STDNAME_TEXT_HEIGHT,PROP_STDTYPE_TEXT_HEIGHT,offsetof(SmallPackage,attrs.height)},
   {"text_colour",PROP_TYPE_COLOUR,offsetof(SmallPackage,attrs.color)},  
   { NULL, 0, 0 },
 };
@@ -238,10 +246,9 @@ smallpackage_draw(SmallPackage *pkg, DiaRenderer *renderer)
   h = elem->height;
   
   renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer_ops->set_linewidth(renderer, SMALLPACKAGE_BORDERWIDTH);
+  renderer_ops->set_linewidth(renderer, pkg->line_width);
   renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
-
   p1.x = x; p1.y = y;
   p2.x = x+w; p2.y = y+h;
 
@@ -266,6 +273,8 @@ smallpackage_draw(SmallPackage *pkg, DiaRenderer *renderer)
   text_draw(pkg->text, renderer);
 
   if ((pkg->st_stereotype != NULL) && (pkg->st_stereotype[0] != '\0')) {
+    renderer_ops->set_font(renderer, pkg->text->font, pkg->text->height);
+
     p1 = pkg->text->position;
     p1.y -= pkg->text->height;
     renderer_ops->draw_string(renderer, pkg->st_stereotype, &p1, 
@@ -360,7 +369,9 @@ smallpackage_create(Point *startpoint,
     pkg->connections[i].connected = NULL;
   }
   pkg->connections[8].flags = CP_FLAGS_MAIN;
-  elem->extra_spacing.border_trans = SMALLPACKAGE_BORDERWIDTH/2.0;
+  pkg->line_width = attributes_get_default_linewidth();
+
+  elem->extra_spacing.border_trans = pkg->line_width/2.0;
 
   pkg->line_color = attributes_get_foreground();
   pkg->fill_color = attributes_get_background();
@@ -392,11 +403,17 @@ smallpackage_destroy(SmallPackage *pkg)
 static DiaObject *
 smallpackage_load(ObjectNode obj_node, int version, const char *filename)
 {
-  return object_load_using_properties(&smallpackage_type,
-                                      obj_node,version,filename);
+  DiaObject *obj = object_load_using_properties(&smallpackage_type,
+                                                obj_node,version,filename);
+  AttributeNode attr;
+  /* For compatibility with previous dia files. If no line_width, use
+   * SMALLPACKAGE_BORDERWIDTH, that was the previous line width.
+   */
+  attr = object_find_attribute(obj_node, PROP_STDNAME_LINE_WIDTH);
+  if (attr == NULL)
+    ((SmallPackage*)obj)->line_width = SMALLPACKAGE_BORDERWIDTH;
+
+  return obj;
 }
 
 
-
-
-
