@@ -21,6 +21,7 @@
 #endif
 
 #include <assert.h>
+#define _BSD_SOURCE 1 /* to get finite */
 #include <math.h>
 
 #include "intl.h"
@@ -122,11 +123,13 @@ static ObjectOps arc_ops = {
   (MoveFunc)            arc_move,
   (MoveHandleFunc)      arc_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
-  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ApplyPropertiesDialogFunc) object_apply_props_from_dialog,
   (ObjectMenuFunc)      NULL,
   (DescribePropsFunc)   arc_describe_props,
   (GetPropsFunc)        arc_get_props,
   (SetPropsFunc)        arc_set_props,
+  (TextEditFunc) 0,
+  (ApplyPropertiesListFunc) object_apply_props,
 };
 
 static PropDescription arc_props[] = {
@@ -151,7 +154,7 @@ arc_describe_props(Arc *arc)
 
 static PropOffset arc_offsets[] = {
   OBJECT_COMMON_PROPERTIES_OFFSETS,
-  { "line_width", PROP_TYPE_REAL, offsetof(Arc, line_width) },
+  { PROP_STDNAME_LINE_WIDTH, PROP_STDTYPE_LINE_WIDTH, offsetof(Arc, line_width) },
   { "line_colour", PROP_TYPE_COLOUR, offsetof(Arc, arc_color) },
   { "line_style", PROP_TYPE_LINESTYLE,
     offsetof(Arc, line_style), offsetof(Arc, dashlength) },
@@ -238,82 +241,18 @@ arc_update_handles(Arc *arc)
   connection_update_handles(conn);
   
   middle_pos = &arc->middle_handle.pos;
+  middle_pos->x = (conn->endpoints[0].x + conn->endpoints[1].x) / 2.0;
+  middle_pos->y = (conn->endpoints[0].y + conn->endpoints[1].y) / 2.0;
 
   dx = conn->endpoints[1].x - conn->endpoints[0].x;
   dy = conn->endpoints[1].y - conn->endpoints[0].y;
   
   dist = sqrt(dx*dx + dy*dy);
   if (dist > 0.000001) {
-    middle_pos->x =
-      (conn->endpoints[0].x + conn->endpoints[1].x) / 2.0 -
-      arc->curve_distance*dy/dist;
-    middle_pos->y =
-      (conn->endpoints[0].y + conn->endpoints[1].y) / 2.0 +
-      arc->curve_distance*dx/dist;
+    middle_pos->x -= arc->curve_distance*dy/dist;
+    middle_pos->y += arc->curve_distance*dx/dist;
   }
 }
-/** returns the number of intersection the circle has with the horizontal line at y=horiz
- * if 1 point intersects then *int1 is that point
- * if 2 points intersect then *int1 and *int2 are these points
- */
-static int
-arc_circle_intersects_horiz(const Arc *arc, real horiz, Point *int1, Point *int2){
-  /* inject y=horiz into r^2 = (x-x_c)^2 + (y-y_c)^2 
-   * this is r^2 = (x-x_c)^2 + (horiz-y_c)^2 
-   * translate to x^2 + b*x + c = 0 
-   * b = -2 x_c 
-   * c = x_c^2 - r^2 + (horiz-y_c)^2 
-   * and solve classically */
-  real b, c, delta;
-  b = -2.0 * arc->center.x;
-  c =  arc->center.x * arc->center.x + (horiz - arc->center.y) * (horiz - arc->center.y) - arc->radius * arc->radius;
-  delta = b*b - 4 * c;
-  if (delta < 0)
-          return 0;
-  else if (delta == 0){
-         int1->x = -b/2; 
-         int1->y = horiz;
-         return 1; 
-  }
-  else {
-         int1->x = (sqrt(delta)-b)/2; 
-         int1->y = horiz;
-         int2->x = (-sqrt(delta)-b)/2; 
-         int2->y = horiz;
-          return 2;
-  }
-}
-/** returns the number of intersection the circle has with the vertical line at x=vert
- * if 1 point intersects then *int1 is that point
- * if 2 points intersect then *int1 and *int2 are these points
- */
-static int
-arc_circle_intersects_vert(const Arc *arc, real vert, Point *int1, Point *int2){
-  /* inject x=vert into r^2 = (x-x_c)^2 + (y-y_c)^2 
-   * this is r^2 = (vert-x_c)^2 + (y-y_c)^2 
-   * translate to y^2 + b*y + c = 0 and solve classically */
-  real b, c, delta;
-  b = -2*arc->center.y;
-  c =  arc->center.y * arc->center.y + (vert - arc->center.x) * (vert - arc->center.x) - arc->radius * arc->radius;
-  delta = b*b - 4 * c;
-  if (delta < 0)
-          return 0;
-  else if (delta == 0){
-         int1->y = -b/2; 
-         int1->x = vert;
-         return 1; 
-  }
-  else {
-         int1->y = (sqrt(delta)-b)/2; 
-         int1->x = vert;
-         int2->y = (-sqrt(delta)-b)/2; 
-         int2->x = vert;
-         return 2;
-  }
-}
-        
-        
-            
 
 static real
 arc_compute_curve_distance(const Arc *arc, const Point *start, const Point *end, const Point *mid)
@@ -370,33 +309,6 @@ arc_find_radial(const Arc *arc, const Point *to, Point *best)
         
 }
 
-/* finds the closest point intersecting the full circle 
- * at any position on the vertical and horizontal lines going through Point to
- * that point is returned in Point *best if 1 is returned */
-static int
-arc_find_closest_vert_horiz(const Arc *arc, const Point *to, Point *best){
-     Point i1,i2;
-     int nh,nv;
-        nh = arc_circle_intersects_horiz(arc, to->y, &i1, &i2);
-        if (nh==2){
-           *best = *closest_to(to,&i1,&i2);
-        }
-        else if (nh==1) {
-           *best = i1;
-        }
-        nv = arc_circle_intersects_vert(arc, to->x, &i1, &i2);
-        if (nv==2){
-           Point tmp;
-           tmp = *closest_to(to,&i1,&i2);
-           *best = *closest_to(to,&tmp,best);
-        }
-        else if (nv==1) {
-           *best = *closest_to(to,&i1,best);
-        }
-        if (nv|nh)
-                return 1;
-        return 0;
-}
 static ObjectChange*
 arc_move_handle(Arc *arc, Handle *handle,
 		Point *to, ConnectionPoint *cp,
@@ -405,6 +317,23 @@ arc_move_handle(Arc *arc, Handle *handle,
   assert(arc!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
+  /* A minimum distance between our three points needs to be maintained
+   * Otherwise our math will get unstable with unpredictable results. */
+  {
+    const Point *p1, *p2;
+    
+    if (handle->id == HANDLE_MIDDLE) {
+      p1 = &arc->connection.endpoints[0];
+      p2 = &arc->connection.endpoints[1];
+    } else {
+      p1 = &arc->middle_handle.pos;
+      p2 = &arc->connection.endpoints[(handle == (&arc->connection.endpoint_handles[0])) ? 1 : 0];
+    }
+    if (   (distance_point_point (to, p1) < 0.01)
+        || (distance_point_point (to, p2) < 0.01))
+      return NULL;
+  }
+
   if (handle->id == HANDLE_MIDDLE) {
           TRACE(printf("curve_dist: %.2f \n",arc->curve_distance));
           arc->curve_distance = arc_compute_curve_distance(arc, &arc->connection.endpoints[0], &arc->connection.endpoints[1], to);
@@ -542,11 +471,11 @@ calculate_arc_object_edge(Arc *arc, real ang_start, real ang_end, DiaObject *obj
 #ifdef TRACE_DIST
   real trace[MAXITER];
   real disttrace[MAXITER];
+  int j = 0;
 #endif
   real mid1, mid2, mid3;
   real dist;
   int i = 0;
-  int j = 0;
 
   mid1 = ang_start;
   mid2 = get_middle_arc_angle(ang_start, ang_end, clockwiseness);
@@ -605,7 +534,6 @@ arc_draw(Arc *arc, DiaRenderer *renderer)
   start_cp = arc->connection.endpoint_handles[0].connected_to;
   end_cp = arc->connection.endpoint_handles[1].connected_to;
 
-  arc_update_data(arc);
   TRACE(printf("drawing arc:\n start:%f :%f,%f \tend:%f :%f,%f\n",arc->angle1,endpoints[0].x,endpoints[0].y, arc->angle2,endpoints[1].x,endpoints[1].y));
 
   if (connpoint_is_autogap(start_cp)) {
@@ -735,9 +663,26 @@ arc_copy(Arc *arc)
   
   newarc->middle_handle = arc->middle_handle;
 
+  arc_update_data(arc);
+
   return &newarc->connection.object;
 }
 
+/* copied from lib/diarenderer.c, the one there should be removed */
+static gboolean
+is_right_hand (const Point *a, const Point *b, const Point *c)
+{
+  Point dot1, dot2;
+
+  dot1 = *a;
+  point_sub(&dot1, c);
+  point_normalize(&dot1);
+  dot2 = *b;
+  point_sub(&dot2, c);
+  point_normalize(&dot2);
+  return point_cross(&dot1, &dot2) > 0;
+}
+
 static void
 arc_update_data(Arc *arc)
 {
@@ -748,6 +693,7 @@ arc_update_data(Arc *arc)
   real x1,y1,x2,y2,xc,yc;
   real lensq, alpha, radius;
   real angle1, angle2;
+  gboolean righthand;
   
   endpoints = &arc->connection.endpoints[0];
   x1 = endpoints[0].x;
@@ -786,37 +732,87 @@ arc_update_data(Arc *arc)
   arc->angle1 = angle1;
   arc->angle2 = angle2;
 
-  extra->start_trans =  (arc->line_width / 2.0);
-  extra->end_trans =     (arc->line_width / 2.0);
-  if (arc->start_arrow.type != ARROW_NONE) 
-    extra->start_trans = MAX(extra->start_trans,arc->start_arrow.width);
-  if (arc->end_arrow.type != ARROW_NONE) 
-    extra->end_trans = MAX(extra->end_trans,arc->end_arrow.width);
-  extra->start_long  = (arc->line_width / 2.0);
+  /* LineBBExtras not applicable to calculate the arrows bounding box */
+  extra->start_trans =
+  extra->end_trans   =
+  extra->start_long  =
   extra->end_long    = (arc->line_width / 2.0);
 
+  /* updates midpoint */
+  arc_update_handles(arc);
+  /* startpoint, midpoint, endpoint */
+  righthand = is_right_hand (&endpoints[0], &arc->middle_handle.pos, &endpoints[1]);
   connection_update_boundingbox(conn);
+
   /* fix boundingbox for arc's special shape XXX find a more elegant way: */
   if (in_angle(0, arc->angle1, arc->angle2)) {
-    obj->bounding_box.right = arc->center.x + arc->radius 
-      + (arc->line_width / 2.0);
+    /* rigth side, y does not matter if included */
+    Point pt = { arc->center.x + arc->radius + (arc->line_width / 2.0), y1 };
+    rectangle_add_point (&obj->bounding_box, &pt);
   }
   if (in_angle(90, arc->angle1, arc->angle2)) {
-    obj->bounding_box.top = arc->center.y - arc->radius
-      - (arc->line_width / 2.0);
+    /* top side, x does not matter if included */
+    Point pt = {x1, arc->center.y - arc->radius - (arc->line_width / 2.0) };
+    rectangle_add_point (&obj->bounding_box, &pt);
   }
   if (in_angle(180, arc->angle1, arc->angle2)) {
-    obj->bounding_box.left = arc->center.x - arc->radius
-      - (arc->line_width / 2.0);
+    /* left side, y does not matter if included */
+    Point pt = { arc->center.x - arc->radius - (arc->line_width / 2.0), y1 };
+    rectangle_add_point (&obj->bounding_box, &pt);
   }
   if (in_angle(270, arc->angle1, arc->angle2)) {
-    obj->bounding_box.bottom = arc->center.y + arc->radius
-      + (arc->line_width / 2.0);
+    /* bootom side, x does not matter if included */
+    Point pt = { x1, arc->center.y + arc->radius + (arc->line_width / 2.0) };
+    rectangle_add_point (&obj->bounding_box, &pt);
+  }
+  if (arc->start_arrow.type != ARROW_NONE) {
+    /* a good from-point would be the chord of arrow length, but draw_arc_with_arrows currently uses the tangent
+     * For big arcs the difference is not huge and the minimum size of small arcs should be limited by the arror length.
+     */
+    Rectangle bbox = {0,};
+    real tmp;
+    Point move_arrow, move_line;
+    Point to = arc->connection.endpoints[0]; 
+    Point from = to;
+    point_sub (&from, &arc->center);
+    tmp = from.x;
+    if (righthand)
+      from.x = -from.y, from.y = tmp;
+    else
+      from.x = from.y, from.y = -tmp;
+    point_add (&from, &to);
+
+    calculate_arrow_point(&arc->start_arrow, &to, &from,
+                          &move_arrow, &move_line, arc->line_width);
+    /* move them */
+    point_sub(&to, &move_arrow);
+    point_sub(&from, &move_line);
+    arrow_bbox(&arc->start_arrow, arc->line_width, &to, &from, &bbox);
+    rectangle_union(&obj->bounding_box, &bbox);
+  }
+  if (arc->end_arrow.type != ARROW_NONE) {
+    Rectangle bbox = {0,};
+    real tmp;
+    Point move_arrow, move_line;
+    Point to = arc->connection.endpoints[1]; 
+    Point from = to;
+    point_sub (&from, &arc->center);
+    tmp = from.x;
+    if (righthand)
+      from.x = from.y, from.y = -tmp;
+    else
+      from.x = -from.y, from.y = tmp;
+    point_add (&from, &to);
+    calculate_arrow_point(&arc->end_arrow, &to, &from,
+                          &move_arrow, &move_line, arc->line_width);
+    /* move them */
+    point_sub(&to, &move_arrow);
+    point_sub(&from, &move_line);
+    arrow_bbox(&arc->end_arrow, arc->line_width, &to, &from, &bbox);
+    rectangle_union(&obj->bounding_box, &bbox);
   }
 
   obj->position = conn->endpoints[0];
-  
-  arc_update_handles(arc);
 }
 
 static void
@@ -833,7 +829,7 @@ arc_save(Arc *arc, ObjectNode obj_node, const char *filename)
 		  arc->curve_distance);
   
   if (arc->line_width != 0.1)
-    data_add_real(new_attribute(obj_node, "line_width"),
+    data_add_real(new_attribute(obj_node, PROP_STDNAME_LINE_WIDTH),
 		  arc->line_width);
   
   if (arc->line_style != LINESTYLE_SOLID)
@@ -885,7 +881,7 @@ arc_load(ObjectNode obj_node, int version, const char *filename)
     arc->curve_distance = data_real(attribute_first_data(attr));
 
   arc->line_width = 0.1;
-  attr = object_find_attribute(obj_node, "line_width");
+  attr = object_find_attribute(obj_node, PROP_STDNAME_LINE_WIDTH);
   if (attr != NULL)
     arc->line_width = data_real(attribute_first_data(attr));
 
@@ -919,3 +915,4 @@ arc_load(ObjectNode obj_node, int version, const char *filename)
 }
 
 
+
