@@ -107,10 +107,7 @@ begin_render(DiaRenderer *self)
                              renderer->dia->bg_color.blue,
                              1.0);
     }
-  cairo_rectangle (renderer->cr, 
-                   renderer->dia->extents.left, renderer->dia->extents.top,
-                   renderer->dia->extents.right, renderer->dia->extents.bottom);
-  cairo_fill (renderer->cr);
+  cairo_paint (renderer->cr);
   if (renderer->with_alpha)
     {
       /* restore to default drawing */
@@ -163,6 +160,18 @@ end_render(DiaRenderer *self)
   DIAG_STATE(renderer->cr)
 }
 
+static void
+ensure_minimum_one_device_unit(DiaCairoRenderer *renderer, real *value)
+{
+  double ax = 1., ay = 1.;
+
+  cairo_device_to_user_distance (renderer->cr, &ax, &ay);
+
+  ax = MAX(ax, ay);
+  if (*value < ax)
+      *value = ax;
+}
+
 static void
 set_linewidth(DiaRenderer *self, real linewidth)
 {  
@@ -170,16 +179,10 @@ set_linewidth(DiaRenderer *self, real linewidth)
 
   DIAG_NOTE(g_message("set_linewidth %f", linewidth));
 
-  /* make hairline? */
-  if (linewidth == 0.0) {
-    double ax = 0.0, ay = 0.0;
-    double bx = 1.0, by = 0.0;    
-    
-    cairo_device_to_user_distance (renderer->cr, &ax, &ay);
-    cairo_device_to_user_distance (renderer->cr, &bx, &by);
-    
-    linewidth = sqrt ((bx - ax) * (bx - ax) + (by - ay) * (by - ay));
-  }
+  /* make hairline? Everythnig below one device unit get the same width,
+   * otherwise 0.0 may end up thicker than 0.0+epsilon
+   */
+  ensure_minimum_one_device_unit(renderer, &linewidth);
 
   cairo_set_line_width (renderer->cr, linewidth);
   DIAG_STATE(renderer->cr)
@@ -284,6 +287,9 @@ set_dashlength(DiaRenderer *self, real length)
 
   DIAG_NOTE(g_message("set_dashlength %f", length));
 
+  /* this call does not make sense, the value is certainly bigger
+   * than one device unit. But the side-effect seems to end the endless loop */
+  ensure_minimum_one_device_unit(renderer, &length);
   renderer->dash_length = length;
 }
 
@@ -301,17 +307,13 @@ set_fillstyle(DiaRenderer *self, FillStyle mode)
   default:
     message_error("DiaCairoRenderer : Unsupported fill mode specified!\n");
   }
-  DIAG_STATE(renderer->cr)
+  DIAG_STATE(DIA_CAIRO_RENDERER (self)->cr)
 }
 
 static void
 set_font(DiaRenderer *self, DiaFont *font, real height)
 {
   DiaCairoRenderer *renderer = DIA_CAIRO_RENDERER (self);
-#ifndef HAVE_PANGOCAIRO_H
-  DiaFontStyle style = dia_font_get_style (font);
-  const char *family_name;
-#endif
   /* pango/cairo wants the font size, not the (line-) height */
   real size = dia_font_get_size (font) * (height / dia_font_get_height (font));
 
@@ -324,18 +326,21 @@ set_font(DiaRenderer *self, DiaFont *font, real height)
   pango_layout_set_font_description (renderer->layout, pfd);
   pango_font_description_free (pfd);
 #else
-  family_name = dia_font_get_family(font);
+  if (renderer->cr) {
+    DiaFontStyle style = dia_font_get_style (font);
+    const char *family_name = dia_font_get_family(font);
 
-  cairo_select_font_face (
-      renderer->cr,
-      family_name,
-      DIA_FONT_STYLE_GET_SLANT(style) == DIA_FONT_NORMAL ? CAIRO_FONT_SLANT_NORMAL : CAIRO_FONT_SLANT_ITALIC,
-      DIA_FONT_STYLE_GET_WEIGHT(style) < DIA_FONT_MEDIUM ? CAIRO_FONT_WEIGHT_NORMAL : CAIRO_FONT_WEIGHT_BOLD); 
-  cairo_set_font_size (renderer->cr, size);
+    cairo_select_font_face (
+        renderer->cr,
+        family_name,
+        DIA_FONT_STYLE_GET_SLANT(style) == DIA_FONT_NORMAL ? CAIRO_FONT_SLANT_NORMAL : CAIRO_FONT_SLANT_ITALIC,
+        DIA_FONT_STYLE_GET_WEIGHT(style) < DIA_FONT_MEDIUM ? CAIRO_FONT_WEIGHT_NORMAL : CAIRO_FONT_WEIGHT_BOLD); 
+    cairo_set_font_size (renderer->cr, size);
+
+    DIAG_STATE(renderer->cr)
+  }
 #endif
 
-  DIAG_STATE(renderer->cr)
-  
   /* for the interactive case we must maintain the font field in the base class */
   if (self->is_interactive) {
     dia_font_ref(font);
@@ -562,6 +567,9 @@ _ellipse(DiaRenderer *self,
   
 #if 1
   cairo_save (renderer->cr);
+  /* don't create a line from the current point to the beginning 
+   * of the ellipse */
+  cairo_new_sub_path (renderer->cr);
   /* copied straight from cairo's documentation, and fixed the bug there */
   cairo_translate (renderer->cr, center->x, center->y);
   cairo_scale (renderer->cr, width / 2., height / 2.);
