@@ -42,15 +42,30 @@ extern "C" {
 }
 #endif
 
+#include "paginate_gdiprint.h"
+
 #if defined HAVE_WINDOWS_H || defined G_OS_WIN32
-namespace W32 {
-// at least Rectangle conflicts ...
-#define WIN32_LEAN_AND_MEAN
-#include <windows.h>
-}
+  namespace W32 {
+   // at least Rectangle conflicts ...
+#  define WIN32_LEAN_AND_MEAN
+#  include <windows.h>
+  }
+#undef STRICT
+typedef W32::HDC HDC;
+typedef W32::HFONT HFONT;
+typedef W32::LOGFONT LOGFONT;
+typedef W32::LOGFONTA LOGFONTA;
+typedef W32::LOGFONTW LOGFONTW;
+
+#include <pango/pangowin32.h>
+
+#elif HAVE_LIBEMF
+  namespace W32 {
+#  include <emf.h>
+  }
 #else
-#include "wmf_gdi.h"
-#define SAVE_EMF
+#  include "wmf_gdi.h"
+#  define SAVE_EMF
 #endif
 
 /* force linking with gdi32 */
@@ -109,6 +124,9 @@ struct _WmfRenderer
     gboolean platform_is_nt; /* advanced line styles supported */
     gboolean target_emf; /* write enhanced metafile */
     W32::RECT margins;
+
+    gboolean use_pango;
+    PangoContext* pango_context;
 };
 
 struct _WmfRendererClass
@@ -139,7 +157,7 @@ UsePen(WmfRenderer* renderer, Color* colour)
     W32::HPEN hOldPen;
     if (colour) {
 	W32::COLORREF rgb = W32COLOR(colour);
-#ifdef G_OS_WIN32
+#if defined(G_OS_WIN32) || HAVE_LIBEMF
 	if ((renderer->platform_is_nt && renderer->hPrintDC) || renderer->target_emf) {
           W32::LOGBRUSH logbrush;
 	  W32::DWORD    dashes[6];
@@ -211,9 +229,13 @@ DonePen(WmfRenderer* renderer, W32::HPEN hPen)
     }
 }
 
-#define DIAG_NOTE /* my_log */
+#ifndef HAVE_LIBEMF
+#  define DIAG_NOTE /* my_log */
+#else
+#  define DIAG_NOTE my_log
+#endif
 void
-my_log(WmfRenderer* renderer, char* format, ...)
+my_log(WmfRenderer* renderer, const char* format, ...)
 {
     gchar *string;
     va_list args;
@@ -300,6 +322,8 @@ end_render(DiaRenderer *self)
 	W32::DeleteEnhMetaFile(hEmf);
     if (renderer->hFont)
 	W32::DeleteObject(renderer->hFont);
+    if (renderer->pango_context)
+        g_object_unref (renderer->pango_context);
 }
 
 static void
@@ -445,44 +469,82 @@ set_font(DiaRenderer *self, DiaFont *font, real height)
     W32::LPCTSTR sFace;
     W32::DWORD dwItalic = 0;
     W32::DWORD dwWeight = FW_DONTCARE;
-    DiaFontStyle style;
+    DiaFontStyle style = dia_font_get_style(font);
+    real font_size = dia_font_get_size (font) * (height / dia_font_get_height (font));
+
 
     DIAG_NOTE(renderer, "set_font %s %f\n", 
               dia_font_get_family (font), height);
-    if (renderer->hFont)
+    if (renderer->hFont) {
 	W32::DeleteObject(renderer->hFont);
-
-    sFace = dia_font_get_family (font);
-    style = dia_font_get_style(font);
-    dwItalic = DIA_FONT_STYLE_GET_SLANT(style) != DIA_FONT_NORMAL;
-
-    /* although there is a known algorithm avoid it for cleanness */
-    switch (DIA_FONT_STYLE_GET_WEIGHT(style)) {
-    case DIA_FONT_ULTRALIGHT    : dwWeight = FW_ULTRALIGHT; break;
-    case DIA_FONT_LIGHT         : dwWeight = FW_LIGHT; break;
-    case DIA_FONT_MEDIUM        : dwWeight = FW_MEDIUM; break;
-    case DIA_FONT_DEMIBOLD      : dwWeight = FW_DEMIBOLD; break;
-    case DIA_FONT_BOLD          : dwWeight = FW_BOLD; break;
-    case DIA_FONT_ULTRABOLD     : dwWeight = FW_ULTRABOLD; break;
-    case DIA_FONT_HEAVY         : dwWeight = FW_HEAVY; break;
-    default : dwWeight = FW_NORMAL; break;
+	renderer->hFont = NULL;
     }
+    if (renderer->pango_context) {
+        g_object_unref (renderer->pango_context);
+	renderer->pango_context = NULL;
+    }
+    
+    if (renderer->use_pango) {
+#ifdef __PANGOWIN32_H__ /* with the pangowin32 backend there is a better way */
+	if (!renderer->pango_context)
+	    renderer->pango_context = pango_win32_get_context ();
 
-    renderer->hFont = (W32::HFONT)W32::CreateFont( 
-	- SC (height),  // logical height of font 
-	0,		// logical average character width 
-	0,		// angle of escapement
-	0,		// base-line orientation angle 
-	dwWeight,	// font weight
-	dwItalic,	// italic attribute flag
-	0,		// underline attribute flag
-	0,		// strikeout attribute flag
-	DEFAULT_CHARSET,	// character set identifier 
-	OUT_TT_PRECIS, 	// output precision 
-	CLIP_DEFAULT_PRECIS,	// clipping precision
-	PROOF_QUALITY,		// output quality 
-	DEFAULT_PITCH,		// pitch and family
-	sFace);		// pointer to typeface name string 
+	PangoFont* pf = pango_context_load_font (renderer->pango_context, dia_font_get_description (font));
+	if (pf)
+	{
+	    W32::LOGFONT* lf = pango_win32_font_logfont (pf);
+	    /* .93 : sligthly smaller looks much better */
+	    lf->lfHeight = -SC(height*.93);
+	    lf->lfHeight = -SC(font_size);
+	    renderer->hFont = (W32::HFONT)W32::CreateFontIndirect (lf);
+	    g_free (lf);
+	    g_object_unref (pf);
+	}
+	else
+	{
+	    gchar *desc = pango_font_description_to_string (dia_font_get_description (font));
+	    message_warning (_("Can not render unknown font:\n%s"), desc);
+	    g_free (desc);
+	}
+#else
+	g_assert_not_reached();
+#endif
+    } else {
+	sFace = dia_font_get_family (font);
+	dwItalic = DIA_FONT_STYLE_GET_SLANT(style) != DIA_FONT_NORMAL;
+
+	/* although there is a known algorithm avoid it for cleanness */
+	switch (DIA_FONT_STYLE_GET_WEIGHT(style)) {
+	case DIA_FONT_ULTRALIGHT    : dwWeight = FW_ULTRALIGHT; break;
+	case DIA_FONT_LIGHT         : dwWeight = FW_LIGHT; break;
+	case DIA_FONT_MEDIUM        : dwWeight = FW_MEDIUM; break;
+	case DIA_FONT_DEMIBOLD      : dwWeight = FW_DEMIBOLD; break;
+	case DIA_FONT_BOLD          : dwWeight = FW_BOLD; break;
+	case DIA_FONT_ULTRABOLD     : dwWeight = FW_ULTRABOLD; break;
+	case DIA_FONT_HEAVY         : dwWeight = FW_HEAVY; break;
+	default : dwWeight = FW_NORMAL; break;
+	}
+	//Hack to get BYTE out of namespace W32
+#       ifndef BYTE
+#       define BYTE unsigned char
+#       endif
+
+	renderer->hFont = (W32::HFONT)W32::CreateFont( 
+		- SC (font_size),  // logical height of font 
+		0,		// logical average character width 
+		0,		// angle of escapement
+		0,		// base-line orientation angle 
+		dwWeight,	// font weight
+		dwItalic,	// italic attribute flag
+		0,		// underline attribute flag
+		0,		// strikeout attribute flag
+		DEFAULT_CHARSET,	// character set identifier 
+		OUT_TT_PRECIS, 	// output precision 
+		CLIP_DEFAULT_PRECIS,	// clipping precision
+		PROOF_QUALITY,		// output quality 
+		DEFAULT_PITCH,		// pitch and family
+		sFace);		// pointer to typeface name string
+    }
 }
 
 static void
@@ -872,10 +934,15 @@ draw_string(DiaRenderer *self,
     hOld = W32::SelectObject(renderer->hFileDC, renderer->hFont);
     {
         // one way to go, but see below ...
-        char* scp; 
+        char* scp = NULL; 
         /* convert from utf8 to active codepage */
         static char codepage[10];
+#ifndef HAVE_LIBEMF
         sprintf (codepage, "CP%d", W32::GetACP ());
+#else
+        /* GetACP() not available in libEMF */
+        sprintf (codepage, "CP1252");
+#endif
 
         scp = g_convert (text, strlen (text),
                          codepage, "UTF-8",
@@ -893,7 +960,7 @@ draw_string(DiaRenderer *self,
             gunichar2* swc = g_utf8_to_utf16 (text, -1, NULL, &wclen, NULL);
             W32::TextOutW (renderer->hFileDC,
                            SCX(pos->x), SCY(pos->y),
-                           swc, wclen);
+                           reinterpret_cast<W32::LPCWSTR>(swc), wclen);
             g_free (swc);
         }
     }
@@ -905,7 +972,7 @@ static void
 draw_image(DiaRenderer *self,
 	   Point *point,
 	   real width, real height,
-	   DiaImage image)
+	   DiaImage *image)
 {
     WmfRenderer *renderer = WMF_RENDERER (self);
 #ifdef SAVE_EMF
@@ -1120,13 +1187,16 @@ wmf_renderer_class_init (WmfRendererClass *klass)
   renderer_class->draw_rect    = draw_rect;
   renderer_class->fill_rect    = fill_rect;
   renderer_class->draw_arc     = draw_arc;
+#ifndef HAVE_LIBEMF
   renderer_class->fill_arc     = fill_arc;
+#endif
   renderer_class->draw_ellipse = draw_ellipse;
   renderer_class->fill_ellipse = fill_ellipse;
 
   renderer_class->draw_string  = draw_string;
+#ifndef HAVE_LIBEMF
   renderer_class->draw_image   = draw_image;
-
+#endif
   /* medium level functions */
   renderer_class->draw_rect = draw_rect;
   renderer_class->draw_polyline  = draw_polyline;
@@ -1136,8 +1206,10 @@ wmf_renderer_class_init (WmfRendererClass *klass)
 #ifndef SAVE_EMF
   renderer_class->fill_bezier   = fill_bezier;
 #endif
+#ifndef HAVE_LIBEMF
   renderer_class->draw_rounded_rect = draw_rounded_rect;
   renderer_class->fill_rounded_rect = fill_rounded_rect;
+#endif
 }
 
 /* plug-in export api */
@@ -1146,7 +1218,7 @@ export_data(DiagramData *data, const gchar *filename,
             const gchar *diafilename, void* user_data)
 {
     WmfRenderer *renderer;
-    W32::HDC  file;
+    W32::HDC  file = NULL;
     W32::HDC refDC;
     Rectangle *extent;
     gint len;
@@ -1173,16 +1245,22 @@ export_data(DiagramData *data, const gchar *filename,
     bbox.bottom = (int)((data->extents.bottom - data->extents.top) * scale *
         100 * W32::GetDeviceCaps(refDC, VERTSIZE) / W32::GetDeviceCaps(refDC, VERTRES));
 
+#if HAVE_LIBEMF
+    FILE* ofile = g_fopen (filename, "w");
+    if (ofile)
+      file = CreateEnhMetaFileWithFILEA (refDC, ofile, &bbox, "Created with Dia/libEMF\0");
+      // file now owned by the metafile DC
+#else
     file = (W32::HDC)W32::CreateEnhMetaFile(
                     refDC, // handle to a reference device context
-#ifdef SAVE_EMF
+#  ifdef SAVE_EMF
                     filename, // pointer to a filename string
-#else
+#  else
                     NULL, // in memory
-#endif
+#  endif
                     &bbox, // pointer to a bounding rectangle
                     "Dia\0Diagram\0"); // pointer to an optional description string 
-
+#endif
     if (file == NULL) {
         message_error(_("Couldn't open: '%s' for writing.\n"), 
 	              dia_message_filename(filename));
@@ -1196,12 +1274,24 @@ export_data(DiagramData *data, const gchar *filename,
     if (user_data == (void*)1) {
 	renderer->target_emf = TRUE;
 	renderer->hPrintDC = 0;
+#ifdef __PANGOWIN32_H__ 
+        renderer->use_pango = TRUE;
+#else
+        renderer->use_pango = FALSE;
+#endif
     } else {
         renderer->hPrintDC = (W32::HDC)user_data;
+        renderer->use_pango = (user_data != NULL); // always for printing
     }
-    /* printing is platform dependent */
-    renderer->platform_is_nt = (W32::GetVersion () < 0x80000000);
 
+    DIAG_NOTE(renderer, "Saving %s:%s\n", renderer->target_emf ? "EMF" : "WMF", filename);
+
+    /* printing is platform dependent */
+#ifdef HAVE_LIBEMF
+    renderer->platform_is_nt = TRUE;
+#else
+    renderer->platform_is_nt = (W32::GetVersion () < 0x80000000);
+#endif
     extent = &data->extents;
 
     /* calculate offsets */
@@ -1210,13 +1300,13 @@ export_data(DiagramData *data, const gchar *filename,
 	renderer->yoff = - data->extents.top;
 	renderer->scale = scale;
     } else {
-        int  ppc = W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALWIDTH) 
-	            / ( data->paper.lmargin + data->paper.width + data->paper.rmargin);
+        int  ppc = (int)(W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALWIDTH) 
+	            / ( data->paper.lmargin + data->paper.width + data->paper.rmargin));
 	/* respect margins */
-	renderer->margins.left   = ppc * data->paper.lmargin - W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALOFFSETX);
-	renderer->margins.top    = ppc * data->paper.tmargin - W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALOFFSETY);
-	renderer->margins.right  = W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALWIDTH) - ppc * data->paper.rmargin;
-	renderer->margins.bottom = W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALHEIGHT) - ppc * data->paper.bmargin;
+	renderer->margins.left   = (int)(ppc * data->paper.lmargin - W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALOFFSETX));
+	renderer->margins.top    = (int)(ppc * data->paper.tmargin - W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALOFFSETY));
+	renderer->margins.right  = (int)(W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALWIDTH) - ppc * data->paper.rmargin);
+	renderer->margins.bottom = (int)(W32::GetDeviceCaps (renderer->hPrintDC, PHYSICALHEIGHT) - ppc * data->paper.bmargin);
 
 	renderer->xoff = - data->extents.left;
 	renderer->yoff = - data->extents.top;
@@ -1242,7 +1332,7 @@ export_data(DiagramData *data, const gchar *filename,
     /* write the placeable header */
     fwrite(&renderer->pmh, 1, 22 /* NOT: sizeof(PLACEABLEMETAHEADER) */, file->file);
 #endif
-    
+
     /* bounding box in device units */
     bbox.left = 0;
     bbox.top = 0;
@@ -1252,9 +1342,11 @@ export_data(DiagramData *data, const gchar *filename,
     /* initialize drawing */
     W32::SetBkMode(renderer->hFileDC, TRANSPARENT);
     W32::SetMapMode(renderer->hFileDC, MM_TEXT);
+#ifndef HAVE_LIBEMF
     W32::IntersectClipRect(renderer->hFileDC, 
                            bbox.left, bbox.top,
                            bbox.right, bbox.bottom);
+#endif
 
     /* write extents */
     DIAG_NOTE(renderer, "export_data extents %f,%f -> %f,%f\n", 
@@ -1285,6 +1377,27 @@ static DiaExportFilter emf_export_filter = {
     "emf"
 };
 
+#ifdef G_OS_WIN32
+static void
+print_callback (DiagramData *data,
+                const gchar *filename,
+		guint        flags,
+		void        *user_data)
+{
+  if (!data)
+    message_error (_("Nothing to print"));
+  else
+    diagram_print_gdi (data, filename);
+}
+
+static DiaCallbackFilter cb_gdi_print = {
+    "FilePrintGDI",
+    N_("Print (GDI) ..."),
+    "/InvisibleMenu/File/FilePrint",
+    print_callback,
+    NULL
+};
+#endif
 
 /* --- dia plug-in interface --- */
 
@@ -1306,6 +1419,11 @@ dia_plugin_init(PluginInfo *info)
      */
     filter_register_export(&wmf_export_filter);
     filter_register_export(&emf_export_filter);
+    
+    filter_register_callback (&cb_gdi_print);
+#elif HAVE_LIBEMF
+    /* not sure if libEMF really saves EMF ;) */
+    filter_register_export(&emf_export_filter);
 #endif
 
     return DIA_PLUGIN_INIT_OK;
